#!/usr/bin/perl
#
#! /usr/bin/env perl
# Script Author: Jeff Oegema, Scionics Computer GmbH, http://www.scionics.de
# Script Name: msparse.pl
# Script Creation Date: 13/5/2002
#
# Script Description:
# A PERL script to parse blase results and add
# color coding. Script is to be within the EMBL
# parsing system and is designed as such.
#
# Note: Script uses <font> tags which are currently
# discouraged by html standards but are simple in this
# case than divs and style sheets
#
# 06/10/02 Added new parse options as requested
#
#
# Code still needs restructuring for cleanliness, but it's functional
#
#

# Include necessary modules

use Getopt::Std;

# Add a few configuration variables

$reference="\n           <b>2.</b> Shevchenko A, Sunyaev S, Loboda A, Shevchenko A, Bork P,\n";
$reference=$reference . "            Ens W, Standing KG. (2001), Anal Chem 73(9),1917-26:\n";
$reference=$reference . "            Charting the proteomes of organisms with unsequenced genomes\n";
$reference=$reference . "            by MALDI-quadrupole time-of-flight mass spectrometry and\n";
$reference=$reference . "            BLAST homology searching\n";
$reference=$reference . "            <a href='http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=PubMed&list_uids=11354471&dopt=Abstract'>Abstract</a>";



$scionicsline="Color Coding and Additional Parsing Added by:\n";
$scionicsline= $scionicsline . "     Jeffrey Oegema, Scionics GmbH\n";
$scionicsline= $scionicsline . "     <a href='http://www.scionics.de'>http://www.scionics.de</a>\n"; 
 
# Step 1: Get the options off the command line
getopt('ipos');

# Step 2: If we don't have the arguments we are supposed
# to, abort here with a message. Can still get errors throug
# of course.  Will do some really basic checking later

if (! defined $opt_i or ! defined $opt_o or ! defined $opt_p or ! defined $opt_s)
{
        print "\nInvalid Options\n\n";
        print "usage: msparse.pl -i <input file> -o <output file> -p <peptides> -s <scoring table>\n\n";
}

# Step 3: We seem to have valid information, lets go on the
# assumption we do and open our conf file and read in the
# information we need, assuming file is there, and in same
# directory as running script

my $dir = $0; $dir =~ s/[^\/]+$//;

#: "/g/bork3/home/yuan/msblast/msblast_1.0/msparse.conf";
open CONFIN, "$dir/msparse.conf";

while (! eof(CONFIN)){

        # Read a line off the file
        $_=<CONFIN>;

        # We have gotten to the 99 scoring table, form the hash
        # Some assumptions here...8 HSP, two tables 99,100

        if (/^99{/) {

                %99;
                $_=<CONFIN>;

                while (! eof(CONFIN) && ! /^}/) {
		
                        /^(\d+)pep\((\S+)\)/;
                        $hashkey= $1 . "pep";
			@fields= split ',', $2;
			$99{$hashkey}= [@fields];
                        $_=<CONFIN>;

                }

        }

        # We have gotten to the 100 scoring table, form the hash

        if (/^100{/) {

                %100;
                $_=<CONFIN>;

                while (! eof(CONFIN) && ! /^}/) {
                        /^(\d+)pep\((\S+)\)/;
                        $hashkey= $1 . "pep";
			@fields= split ',', $2;
			$100{$hashkey}= [@fields];
			#print "@{$100{$hashkey}}\n";
                        $_=<CONFIN>;
                }

        }
        # Get the colors

        if (/^positive=#(\w+)/) { $positivecolor= "#" . $1;}
        if (/^negative=#(\w+)/) { $negativecolor= "#" . $1;}
        if (/^borderline=#(\w+)/) { $borderlinecolor= "#" . $1;}
        if (/^bordernumber=(\w+)/) { $bordernumber=$1;}
}

close CONFIN;

$colorline="Color Key: <font color='$positivecolor'>red = positive hit;</font><font color='$borderlinecolor'> green = borderline hit;</font> <font color='$negativecolor'>black = negative result</font><font color='black'>";

# Step 4, we are done getting configuration information.  Lets take in the
# file to be parsed and do what's necessary with it

open PARSEIN, $opt_i or die "can't read from file $opt_i [$!]\n";

# This was once more complex..going to do a flag based parsing method
# for now, yields simpler looking code 



$status=0;
%hitlist;	  # This will become the list at the top
@top;	 	  # This will become the lines before the list at the top
@middle;	  # This will become the lines after the list, before the entries
@end;		  # This will become the text at the bottom of the file
%entries;	  # This will become the entries	

while (! eof(PARSEIN)){
	$_=<PARSEIN>;

	if(($status==2)&&(/^WARN/||/^Parameters/)){
		$status=3;
		push @end,$_;
	}elsif ($status==3) {
		push @end,$_;
	}


	# Match the start of the entries
	# For users' own db in FASTA format >id descript...
	if (/^>\/:(\S+)/) {
	#if (/^>\/?:?(\S+)/) {
		#print "$1\n";
		$key=$1;
		push @{$entries{$key}}, $_;
		$status=2;
	} elsif ($status==2) {
		push @{$entries{$key}}, $_;
	}


	# Match the key lines at the top and put them in a hash
	# For users' own db in FASTA format >id descript...
	if( /^\/:(\S+)(.*)$/) { 
	#if( /^\/?:?(\S+)(.*)$/) { 
		$hitlist{$1}=$_;
		$status=1;
	} elsif ($status==1) {
		push @middle, $_;
	}


	# If nothing else has changed put the lines into the "top" array
	if($status==0){
		push @top, $_;	
	}
}


close PARSEIN;

# O.K Data file read in, parsed into little easy to handle chunks
# Lets build the scoring information for each entry
# This looks a bit bizarre but it builds an array of scores for
# each entry and puts the total score in the 0 position of the array

%scores;
foreach $entrykey (keys %entries) {

	@work=@{$entries{$entrykey}};
	$tempscore=0;
	push @{$scores{$entrykey}}," ";

	foreach (@work){

		/^\s+Score = (\d+)/ && do {
			$tempscore=$tempscore+$1;
			push @{$scores{$entrykey}},$tempscore;
		}
	}

	@{$scores{$entrykey}}[0]=$tempscore;
}


# So now we have all the scores if life is good, all the entryies by name,
# So lets just do the final analysis

#first figure out which score array we are using the check this


# Addition, $opt_p needs to be interpolated if its in certain ranges
if($opt_p >=20 && $opt_p <=24){
	$pvalue=20;
}elsif ($opt_p >=25 && $opt_p <=34){
	$pvalue=30;
}elsif ($opt_p >=35 && $opt_p <=44){
	$pvalue=40;
}elsif ($opt_p >=45 && $opt_p <=50){
	$pvalue=50;
}else{
	$pvalue = $opt_p;
}


if($opt_s==99){
	$scoretable=99; 
	#print "Scoring with 99\n";
	$mykey=$pvalue . "pep";
	@scoringrow=@{$99{$mykey}};
	if(scalar(@scoringrow)<1){ print "Illegal Peptide Value.\n";exit;}
}elsif($opt_s==100){
	#print "Scoring with 100\n";
	$scoretable=100; 
	$mykey=$pvalue . "pep";
	@scoringrow=@{$100{$mykey}};
	if(scalar(@scoringrow)<1){ print "Illegal Peptide Value.\n";exit;}

}else{
	print "Illegal Scoring Table Value.\n";
	exit;
}



# The scoring row now nicely lies in the scoring row array so lets do
# the nifhty little analysis.

@positives;
@negatives;
@borderlines;

foreach $scorekey (keys %scores){
	$positive=0;

	#Pull the score array from the entry
	@scorework=@{$scores{$scorekey}};

	for ($count=1; $count < scalar(@scorework) ; $count++){
		#print "Comparing $scorework[$count] and $scoringrow[$count-1]\n";
		if ($scorework[$count] >= $scoringrow[$count-1]){
			$positive=3;
		}elsif ($scorework[$count] + $bordernumber >= $scoringrow[$count-1]){
			if ($positive <3){ $positive=2;}
		}

		# Deal with the case where the result has more positive entries than
		# 8, or more results than the table has and make positive

		if (scalar(@scorework) > 9){
			$positive=3;
		}elsif (scalar(@scorework) > scalar(@scoringrow)+1){
			$positive=3;
		}
	
	}

	if ($positive==3){
		push @positives, $scorekey;
		#$positives{$scorekey}=1;
		}
		#print "$scorekey was POSITIVE\n";
	elsif ($positive==2){
		push @borderlines, $scorekey;
		#$borderlines{$scorekey}=1;
		}
		#print "$scorekey was BORDERLINE\n";
	else {
		push @negatives, $scorekey; 
		#$negatives{$scorekey}=1;
		#print "$scorekey was NEGATIVE\n";
	}

}







# Additions made here.  Parse at top to chenge the header were it says "Smallest
# Sum Probability to now say total score
# These are simple substitutions in order to replace labels and such
foreach (@top) {
	s/Smallest//;
	s/Sum//;
	
	s/Probability/Total/;
	s/P\(N\)\s+N/ Score/;
	
	# Add the color listing
	s/(.*High.*Total.*$)/$colorline\n\n\1/;


	# Add the reference information
	s/(.*Reference:.*$)/\1\n$reference/;
}



# This next section reparses the hitlist entries really quick to put the total
# score on them instead of the p value

foreach $key (keys %hitlist){
	$replace=@{$scores{$key}}[0];
	$hitlist{$key}=~s/(\d|\.|e|-)+\s+(\d+)\s*$/$replace\n/;
} 


# Go replace the expect and sum p for the entries as well
# The @{$entries{$key}} references the entries array and
# it gets modified directly by the code

foreach $key (keys %entries){

	$status=0;

	$replace=@{$scores{$key}}[0];
	
	foreach(@{$entries{$key}}){

		/.*Expect.*/ && do {

			s/,.*Expect.*$//;
			if ($status==0){
				$_="Total Score: $replace\n\n" . $_;
				$status=1;
			}
		}
	}
}





# Well, we know the positives and negatives so lets resort and print the file

# Alrighty all adjustments made, lets put out the outfile
open OUTFILE, ">$opt_o" or die "can't write to file $opt_o [$!]\n";

print OUTFILE @top;

# Sort the positives, color them and print them out
# Scores is a hash containing arrays of scores.  This sort
# uses the keys in the positive,  negative, and borderline arrays
# to retrieve the total scores that were stored in the arrays

@positivesort= sort {@{$scores{$b}}[0] <=> @{$scores{$a}}[0]} @positives;
@borderlinesort= sort {@{$scores{$b}}[0] <=> @{$scores{$a}}[0]} @borderlines;
@negativesort= sort {@{$scores{$b}}[0] <=> @{$scores{$a}}[0]} @negatives;

print OUTFILE "<font color='" . $positivecolor . "'>\n";

foreach $entry (@positivesort) {
	print OUTFILE  $hitlist{$entry};
	#print $hitlist{$entry};
	#print "@{$scores{$entry}}[0]\n";
}

print OUTFILE "</font><font color='black'>\n";
print OUTFILE "<font color='" . $borderlinecolor . "'>\n";

foreach $entry (@borderlinesort) {
	print OUTFILE $hitlist{$entry};
	#print "@{$scores{$entry}}[0]\n";
	#print "$entry\n";
}

print OUTFILE "</font><font color='black'>\n";
print OUTFILE "<font color='" . $negativecolor . "'>\n";

foreach $entry (@negativesort) {
	print OUTFILE $hitlist{$entry};
	#print "@{$scores{$entry}}[0]\n";
	#print "$entry\n";
}
print OUTFILE "</font><font color='black'>\n";


print OUTFILE @middle;


print OUTFILE "<font color='" . $positivecolor . "'>\n";

foreach $entry (@positivesort) {

	print OUTFILE @{$entries{$entry}};

}

print OUTFILE "</font>\n";


print OUTFILE "<font color='" . $borderlinecolor . "'>\n";

foreach $entry (@borderlinesort) {

	print OUTFILE @{$entries{$entry}};

}

print OUTFILE "</font>\n";


print OUTFILE "<font color='" . $negativecolor . "'>\n";

foreach $entry (@negativesort) {

	print OUTFILE @{$entries{$entry}};

}

print OUTFILE "</font>\n";

print OUTFILE @end;

# Print the scoring table used
print OUTFILE "\n\nAdditional Parsing Information\n-------------------------------\n";
print OUTFILE "\nScoring Table Used $scoretable, $pvalue peptides\n\n";
$count=1;

print OUTFILE "HSP	Score\n";
print OUTFILE "----------------\n";
foreach $value (@scoringrow){
	print OUTFILE "$count	$value\n";
	$count=$count+1;
}
print OUTFILE "\n\n";

print OUTFILE $scionicsline;
print OUTFILE "\n\n";
close OUTFILE;





