#!/usr/bin/perl -w

# yuan@embl.de
# Thu Jul 17 18:16:07 CEST 2003

use Getopt::Long;

my ($help, $verbose, $debug, $output, $dir, $wudir, $db, $query_file, $uniq_pep, $score_table);

GetOptions(
	       "verbose"       => \$verbose,
           "debug"         => \$debug,
		   "output:s"      => \$output,
		   "dir=s"         => \$dir,
		   "wudir=s"       => \$wudir,
		   "db=s"          => \$db,
		   "query=s"       => \$query_file,
		   "uniq_pep:i"    => \$uniq_pep,
		   "score_table:i" => \$score_table,
		   "help"          => \$help,
		   );

#if ($help or ! @ARGV) {
if ($help) {
	print <<END;

MSBLAST (vers. 1.0)
  developed by yuan\@embl.de, Bork Group, EMBL, Heidelberg. Jul.2003.

  References: 
    1. URL: MSBLAST search service: 
      http://www.bork.embl.de/Blast2/msblast.html

    2. Shevchenko, A., Sunyaev, S., Loboba, A., Shevchenko, A., Bork, P., Ens, 
      W., & Standing, K.G., 2001:
      Charting the proteomes of organisms with unsequenced genomes by MALDI-
      Quadrupole time-of fligh mass spectrometry and BLAST homology searching.
      Anal. Chem., 2001 May 1; 73(9):1917-26.

  
$0: 
This is the Mass Spectrometry driven BLAST (ref. 1 & 2).
This program runs a single wublastp search and outputs the result in html.
It requires that a query file in FASTA and a database prepared for WUBLASTP search 
are given and the unique peptides are known.

Options:

   -help        this text
   
   -db          the database name prepared for a WUBLASTP search (refer to 'setdb')
   
   -dir         the directory where run_msblast.pl script (and other scripts) resides. If it is 
                in your PATH, it\'ll set that automatically.

   -query       query_file name

   -score_table the score table, default 100

   -uniq_pep    the unique peptides, default 1

   -wudir       this directory needs to be set for the location of your blastp 
                program (WUBLAST) if it is different than your directory specified 
                by -dir. Otherwise it takes over the directory given by -dir <dir>.

   -output      a file name for your output. If not specified, the output will 
                be stored under a filename consisting of the basic filename of 
                the query file with a appendix ".html".

For example:

  run_blast.pl -db nrdb95 -query example1.seq -uniq_pep 5

END

  exit(1);
}

unless ($dir) { 
  $dir = $0 ;
  $dir =~ s/[^\/]+$//;
}

$wudir = $wudir ? $wudir : "$dir";

my $file_base  = $query_file; 
   $file_base  =~ s/\.\w+$//;

$uniq_pep    = $uniq_pep    ? $uniq_pep    :                   1;         # default 1;
$score_table = $score_table ? $score_table :                 100;         # default 100;
$output      = $output      ? $output      : "${file_base}.html";

# Run the WUBLAST
print "Running $wudir/blastp $db \"$query_file\" E=100 V=50000 B=50000 -stats -qtype -qres -nogaps -filter none -matrix PAM30MS -nogap -hspmax 100 10-sort_by_totalscore -span1 >  \"${file_base}.blp\"\n\n" if $verbose;

system("$wudir/blastp $db \"$query_file\" E=100 V=50000 B=50000 -stats -qtype -qres -nogaps -filter none -matrix PAM30MS -nogap -hspmax 100 -sort_by_totalscore -span1 >  \"${file_base}.blp\"");


# Parsing the WUBLAST result
print "Running $dir/msparse.pl -i \"${file_base}.blp\" -o \"${file_base}.new\" -p $uniq_pep -s $score_table\n\n" if $verbose;

system("$dir/msparse.pl -i \"${file_base}.blp\" -o \"${file_base}.new\" -p $uniq_pep -s $score_table");

# Producing the html result.
print "Running $dir/eval_msblast.pl \"${file_base}.new\" > $output\n\n" if $verbose;

system("$dir/eval_msblast.pl \"${file_base}.new\" > \"$output\"");


# Cleaning up
#print "Deleting the file \"${file_base}.blp\"\n\n" if $verbose;
#unlink("${file_base}.blp");

#print "Deleting the file \"${file_base}.new\"\n\n" if $verbose;
#unlink("${file_base}.new");

exit(0);
